/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.json;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import java.io.IOException;
import javax.security.auth.x500.X500Principal;
import net.java.truelicense.core.License;
import net.java.truelicense.core.auth.*;
import net.java.truelicense.core.codec.*;
import net.java.truelicense.json.codec.JsonCodec;

/**
 * A repository which uses a {@link JsonCodec} to encode/decode
 * {@link License}s for signing/verifying.
 * This type of repository is used in V2/JSON format license keys.
 *
 * @author Christian Schlichtherle
 */
public class JsonLicenseRepository extends BasicRepository {

    static final Module licenseModule;

    static {
        final SimpleModule module = new SimpleModule();
        // A custom serializer is required because X500Principal does
        // not have a default constructor.
        module.addSerializer(new StdSerializer<X500Principal>(X500Principal.class) {
            @Override public void serialize(
                    X500Principal value,
                    JsonGenerator jgen,
                    SerializerProvider provider)
            throws IOException {
                jgen.writeString(value.getName());
            }
        });
        licenseModule = module;
    }

    private final Codec codec;

    public JsonLicenseRepository() {
        final ObjectMapper mapper = new ObjectMapper();
        mapper.setSerializationInclusion(JsonInclude.Include.NON_DEFAULT);
        mapper.registerModule(licenseModule);
        codec = new JsonCodec(mapper, License.class);
    }

    @Override public Codec codec() { return codec; }
}
